import { capitalizeName, indent } from '../../utils.js';
import { createFileMetadataString, escapeCppName } from '../helpers.js';
export function getSwiftVariantTypename(variant) {
    const types = variant.variants.map((t) => escapeCppName(capitalizeName(t.getCode('swift'))));
    return `Variant_${types.join('_')}`;
}
export function getSwiftVariantCaseName(type) {
    const code = type.getCode('swift');
    return `some${capitalizeName(code)}`;
}
export function createSwiftVariant(variant) {
    const typename = getSwiftVariantTypename(variant);
    const cases = variant.variants
        .map((t) => {
        const type = t.getCode('swift');
        return `case ${getSwiftVariantCaseName(t)}(${type})`;
    })
        .join('\n');
    const jsSignature = variant.variants.map((t) => t.kind).join(' | ');
    const code = `
${createFileMetadataString(`${typename}.swift`)}

/**
 * An Swift enum with associated values representing a Variant/Union type.
 * JS type: \`${jsSignature}\`
 */
@frozen
public enum ${typename} {
  ${indent(cases, '  ')}
}
  `.trim();
    return {
        content: code,
        language: 'swift',
        name: `${typename}.swift`,
        platform: 'ios',
        subdirectory: [],
    };
}
